@@ -1,14 +1,15 @@
 import json
 import copy
+from urllib import parse
 from TDhelper.network.http.REST_HTTP import GET, POST
-
+from TDhelper.Decorators.log import logging, logging_setup
 
 class RPC:
     '''
         RPC. 此类只能配合webservice/rpc使用, 独立使用将会报错.
     '''
 
-    def __init__(self, service_center_uri, secret):
+    def __init__(self, service_center_uri, secret, logging_config= None):
         '''
             初始化
             - params:
@@ -19,6 +20,17 @@ class RPC:
         self._sc_uri = service_center_uri
         self._apisTable = {}
         self._current_service= None
+        self._logging= False
+        if logging_config:
+            self._logging= True
+            logging_setup(logging_config)
+
+    def _log(self, msg, type="info"):
+        if self._logging:
+            if type.lower()=="info":
+                logging.info(msg)
+            elif type.lower()=="error":
+                logging.error(msg)
 
     def _getApi(self, service, method):
         if service+method not in self._apisTable:
@@ -137,34 +149,79 @@ class RPC:
                                 return False, m_msg
                             else:
                                 if ret:
-                                    params = methods[i]['params']
-                                    m_method_id = ret['msg']["id"]
-                                    if params:
-                                        # 有参数能注册.
-                                        for param_offset in range(0, len(params)):
-                                            params[param_offset]['serviceUri'] = m_method_id
-                                            m_service_method_params_post_data = ''
-                                            m_count = 0
-                                            for k, v in params[param_offset].items():
-                                                if not v:
-                                                    raise Exception(
-                                                        'methods params (%s) is none.' % k.lower())
-                                                m_service_method_params_post_data += k + \
-                                                    "=" + str(v)
-                                                m_count += 1
-                                                try:
-                                                    if m_count < len(params[param_offset]):
-                                                        m_service_method_params_post_data += "&"
-                                                except Exception as e:
-                                                    raise e
-                                            state, ret = POST(
-                                                self._sc_uri+"params/", post_data=bytes(m_service_method_params_post_data, encoding='utf-8'), time_out=15)
+                                    if 'params' in methods[i]:
+                                        params = methods[i]['params']
+                                        m_method_id = ret['msg']["id"]
+                                        if params:
+                                            # 有参数能注册.
+                                            for param_offset in range(0, len(params)):
+                                                params[param_offset]['serviceUri'] = m_method_id
+                                                m_service_method_params_post_data = ''
+                                                m_count = 0
+                                                for k, v in params[param_offset].items():
+                                                    if not v:
+                                                        raise Exception(
+                                                            'methods params (%s) is none.' % k.lower())
+                                                    m_service_method_params_post_data += k + \
+                                                        "=" + str(v)
+                                                    m_count += 1
+                                                    try:
+                                                        if m_count < len(params[param_offset]):
+                                                            m_service_method_params_post_data += "&"
+                                                    except Exception as e:
+                                                        raise e
+                                                state, ret = POST(
+                                                    self._sc_uri+"params/", post_data=bytes(m_service_method_params_post_data, encoding='utf-8'), time_out=15)
+                                    # 注册返回值
+                                    if 'returns' in methods[i]:
+                                        m_returns= methods[i]['returns']
+                                        if m_returns:
+                                            # 有返回值设置进行注册.
+                                            reg_returns_params=''
+                                            m_count=0
+                                            m_returns['serviceUri']= m_method_id
+                                            for k,v in m_returns.items():
+                                                if k.lower()!= 'descriptions':
+                                                    reg_returns_params+= k+"="+parse.quote(str(v))
+                                                    m_count+=1
+                                                    if 'descriptions' in m_returns:
+                                                        if m_count< len(m_returns)-1:
+                                                            reg_returns_params+="&"
+                                                    else:
+                                                        if m_count < len(m_returns):
+                                                            reg_returns_params+="&"
+                                            state, ret= POST(self._sc_uri+"returns/",post_data= bytes(reg_returns_params,encoding='utf-8'),time_out=15)
+                                            if state == 200:
+                                                ret = json.loads(str(ret, encoding='utf-8'))
+                                                if ret['state'] != 200:
+                                                    m_msg = 'register service methods failed. http code(%d), msg:%s' % (ret['state'], ret['msg'])
+                                                    return False, m_msg
+                                                else:
+                                                    m_returns_id= ret['msg']['id']
+                                                    if 'descriptions' in m_returns:
+                                                        # 注册返回值说明
+                                                        for item in m_returns['descriptions']:
+                                                            m_returns_description= 'returns='+ str(m_returns_id)
+                                                            m_returns_description+= '&key='+parse.quote(item['key'])
+                                                            m_returns_description+= '&valueDescription='+parse.quote(item['valueDescription'])
+                                                            state, ret= POST(self._sc_uri+ "returnDescriptons/", post_data=bytes(m_returns_description,encoding='utf-8'),time_out=15)
+                                                            if state!=200:
+                                                                m_msg = 'register return description failed. http code(%d)'% state
+                                                                return False, m_msg
+                                                            else:
+                                                                ret = json.loads(str(ret, encoding='utf-8'))
+                                                                if ret['state']!=200:
+                                                                    m_msg= 'register return description failed. state(%d), msg(%s), key()'%(ret['state'],ret['msg'], item['key'])
+                                                                    return False, m_msg
+                                            else:
+                                                m_msg = 'register returns failed. http code(%d)'% state
+                                                return False, m_msg
+
                                 else:
                                     if ret['state'] != 200:
                                         m_msg = 'register service hosts failed. http code(%d), msg:%s' % (ret[
                                             'state'], ret['msg'])
                                         return False, m_msg
-
                     return True, 'register service success.'
                 else:
                     msg = "register service http error. http code(%s), msg:%s" % (ret[
@@ -224,6 +281,7 @@ class RPC:
                 try:
                     if m_method == u"GET":
                         m_uri += "?" + params_data
+                        self._log(m_uri)
                         state, ret = GET(m_uri, time_out=15)
                         if state == 200:
                             try:
@@ -238,6 +296,7 @@ class RPC:
                             except:
                                 return {"httpCode": ret['state'], "data": ret}
                     elif m_method == u"POST":
+                        self._log(m_uri+", postdata:%s"%params_data)
                         state, ret = POST(m_uri, bytes(
                             params_data, encoding='utf-8'), time_out=15)
                         if state == 200:
@@ -280,183 +339,59 @@ if __name__ == '__main__':
     #print(m_rpc.call("remote_config","remote_config_register", **{"name":"测试远程配置服务注册","serivce_key":"test_rpc_register"}))
     '''
     serviceConfig = {
-        "services": [
-            {
-                # 服务配置
-                "service": {
-                    "name": "远程配置中心服务",
-                    "description": "远程序配置服务.",
-                    "key": "394830498309480398403940394",
-                    "httpProtocol": "http://"
-                },
-                # 服务器配置
-                "hosts": [
-                    {
-                        "host": "127.0.0.1",
-                        "port": 8001
-                    }
-                ],
-                # 方法配置
-                "methods": [
-                    {
-                        "key": "REMOTE_CONFIG_SERVICE_LIST",
-                        "uri": "api/services/",
-                        "method": "GET",
-                        "version": "1.0.0",
-                        "description": "获取服务列表",
-                        "params": []
-                    },
-                    {
-                        "key": "REMOTE_CONFIG_SERVICE_CREATE",
-                        "uri": "api/services/",
-                        "method": "POST",
-                        "version": "1.0.0",
-                        "description": "向配置中心注册一个服务.",
-                        "params": [
+    "services": [
+        {
+            # 服务配置
+            "service": {
+                "name": "TEST",
+                "description":"TEST",
+                "key":"TEST",
+                "accessSecret": "slkjflkjdfejfoejfoefef",
+                "httpProtocol":"http://"
+            },
+            # 服务器配置
+            "hosts":[
+                {
+                    "host": "127.0.0.1",
+                    "port": 8004
+                }
+            ],
+            # 方法配置
+            "methods":[
+                # 帐号相关 begin
+                {
+                    "key": "TEST" + "_ACCOUNTS_LIST",
+                    "uri": "api/accounts/",
+                    "method": "GET",
+                    "version": "1.0.0",
+                    "description": "获取帐号列表",
+                    "params": [
+                        {
+                            "key":"page",
+                            "descript":"页码",
+                            "defaultValue": "1"
+                        }
+                    ],
+                    "returns":{
+                        "valueType":"json",
+                        "examples": json.dumps({"state":200,"msg":{"count": 0,"next": None,"previous": None,"results": []}}),
+                        "descriptions":[
                             {
-                                "key": "name",
-                                "description": "名称"
+                                "key":"state",
+                                "valueDescription":"200:success,\r\n-1:failed."
                             },
                             {
-                                "key": "service_key",
-                                "description": "服务索引"
-                            }
-                        ]
-                    },
-                    {
-                        "key": "REMOTE_CONFIG_SERVICE_GETBYID",
-                        "uri": "api/services/{pk}/",
-                        "method": "GET",
-                        "version": "1.0.0",
-                        "description": "根据ID获取一个服务.",
-                        "params": [
-                            {
-                                "key": "pk",
-                                "description": "ID值"
-                            }
-                        ]
-                    },
-                    {
-                        "key": "REMOTE_CONFIG_SERVICE_UPDATE",
-                        "uri": "api/services/{pk}/",
-                        "method": "POST",
-                        "version": "1.0.0",
-                        "description": "更新一个服务.",
-                        "params": [
-                            {
-                                "key": "pk",
-                                "description": "ID值"
-                            },
-                            {
-                                "key": "name",
-                                "description": "名称"
-                            },
-                            {
-                                "key": "service_key",
-                                "description": "服务索引"
-                            }
-                        ]
-                    },
-                    {
-                        "key": "REMOTE_CONFIG_SERVICE_DELETE",
-                        "uri": "api/services/{pk}/",
-                        "method": u"DELETE",
-                        "version": "1.0.0",
-                        "description": "根据ID删除一个服务.",
-                        "params": [
-                            {
-                                "key": "pk",
-                                "description": "ID值"
-                            }
-                        ]
-                    },
-                    {
-                        "key": "SERVICE_REMOTE_CONFIG_LIST",
-                        "uri": "api/configs/",
-                        "method": "GET",
-                        "version": "1.0.0",
-                        "description": "获取配置列表",
-                        "params": []
-                    },
-                    {
-                        "key": "SERVICE_REMOTE_CONFIG_REGISTER",
-                        "uri": "api/configs/",
-                        "method": "POST",
-                        "version": "1.0.0",
-                        "description": "注册一个配置变量",
-                        "params": [
-                            {
-                                "key": "config_key",
-                                "description": "配置索引."
-                            }
-                        ]
-                    },
-                    {
-                        "key": "SERVICE_REMOTE_CONFIG_UPDATE",
-                        "uri": "api/configs/{pk}",
-                        "method": "PUT",
-                        "version": "1.0.0",
-                        "description": "更新一个配置",
-                        "params": [
-                            {
-                                "key": "pk",
-                                "description": "ID值"
-                            },
-                            {
-                                "key": "config_key",
-                                "description": "配置索引."
-                            },
-                            {
-                                "key": "config_value",
-                                "description": "配置的值."
-                            }
-                        ]
-                    },
-                    {
-                        "key": "SERVICE_REMOTE_CONFIG_GET_BY_ID",
-                        "uri": "api/configs/{pk}/",
-                        "method": "GET",
-                        "version": "1.0.0",
-                        "description": "根据ID获取一个配置.",
-                        "params": [
-                            {
-                                "key": "pk",
-                                "description": "ID值"
-                            }
-                        ]
-                    },
-                    {
-                        "key": "SERVICE_REMOTE_CONFIG_DELETE",
-                        "uri": "api/configs/{pk}/",
-                        "method": u"DELETE",
-                        "version": "1.0.0",
-                        "description": "根据ID删除一个配置.",
-                        "params": [
-                            {
-                                "key": "pk",
-                                "description": "ID值"
-                            }
-                        ]
-                    },
-                    {
-                        "key": "SERVICE_REMOTE_CONFIG_GETBYKEY",
-                        "uri": "api/configs/getConfigsByServiceKey/",
-                        "method": "GET",
-                        "version": "1.0.0",
-                        "description": "根据服务KEY获取配置项",
-                        "params": [
-                            {
-                                "key": "key",
-                                "description": "根据服务KEY获取此服务的所有配置项."
+                                "key":"msg",
+                                "valueDescription":"success returns json({count:<int>(total records count),next:<string>(next page url),previous:<string>(previous page url),results:[json(<account>)]}),\r\nfailed returns error msg(<string>)."
                             }
                         ]
                     }
-
-                ]
-            }
-        ]
+                }
+            ]
+        }
+    ]
     }
-    '''
+
     count=0
     for item in serviceConfig["services"]:
         print(m_rpc.register(
@@ -464,9 +399,18 @@ if __name__ == '__main__':
             hosts=item['hosts'],
             methods=item['methods']))
     '''
-    g=m_rpc.handle("REMOTE_CONFIG")
-    ret= g.call("REMOTE_CONFIG_SERVICE_CREATE",**{"name":'ggg',"service_key":"cccc"})
+    g=m_rpc.handle("PERMISSION_CENTER")
+    ret= g.call(
+        "PERMISSION_CENTER_PERMISSIONS_CREATE",
+        **{
+            "permission_key":"COUNTRY_CREATE",
+            "permission_name":"新建一个国家",
+            "permission_domain":"http://127.0.0.1:8003",
+            "permission_uri":"/api/countrys",
+            "permission_enable": 1
+        })
     print(ret)
-    m_id= ret['data']['id']
-    ret=g.call('REMOTE_CONFIG_CONFIG_REGISTER',**{"config_key":"test1","config_value":"testvalue","service":m_id})
-    print(ret)
\ No newline at end of file
+    #m_id= ret['data']['id']
+    #ret=g.call('REMOTE_CONFIG_CONFIG_REGISTER',**{"config_key":"test1","config_value":"testvalue","service":m_id})
+    #print(ret)
+    '''
\ No newline at end of file
